@@ -88,6 +88,7 @@ class Propheto:
         self.deployment = object
         self.code_introspecter = CodeIntrospect(file_dir=file_dir, **kwargs)
         self.working_directory = file_dir
+        self.package_dir = file_dir
         self.parent_dir, self.project_dir = self._generate_base_artifacts()
 
     def __repr__(self) -> str:
@@ -226,10 +227,13 @@ class Propheto:
                 raise Exception("Project name is a required field.")
 
     def _create_directory(self) -> str:
-        DIR_PATH = Path(self.working_directory, "propheto-package")
-        DIR_PATH.mkdir(parents=True, exist_ok=True)
-        os.chdir(DIR_PATH)
-        return DIR_PATH.absolute()
+        """
+        Create a directory for the generated service code to reside if one doesnt already exist.
+        """
+        self.package_dir = Path(self.working_directory, "propheto-package")
+        self.package_dir.mkdir(parents=True, exist_ok=True)
+        os.chdir(self.package_dir)
+        return self.package_dir.absolute()
 
     def model(self, action: str, *args, **kwargs) -> None:
         """
@@ -387,6 +391,7 @@ class Propheto:
         target: str,
         deployment_type: Optional[str] = "realtime-serverless",
         if_exists: Optional[str] = "update",
+        introspect_code: Optional[bool] = False,
         *args,
         **kwargs,
     ) -> None:
@@ -403,13 +408,17 @@ class Propheto:
                 Deployment type for the service. 
         if_exists : str, optional
                 Determine what to do if resource already exists
+        introspect_code : bool, optional
+                Flag to determine if code introspection should occur
         """
         # Read notebook
         # TODO: generalize to read code not notebook
-        self.code_introspecter.get_notebook_details()
-        _ = self.code_introspecter.read_notebook()
-        _ = self.code_introspecter.get_notebook_code_cells()
+        if introspect_code:
+            self.code_introspecter.get_notebook_details()
+            _ = self.code_introspecter.read_notebook()
+            _ = self.code_introspecter.get_notebook_code_cells()
         self._validate_target(target)
+        os.chdir(self.package_dir)
         if target == "aws":
             self.deployment = AWS(**kwargs)
             self._deploy_aws(model, action="deploy")
@@ -562,11 +571,12 @@ class Propheto:
         )
 
         # CREATE BUCKET & FORMAT NAME
-        s3_bucket_name = self.project_name.replace(" ", "")
+        s3_bucket_name = self.project_name.replace(" ", "").replace(".", "").replace('_', '')
         for number in range(10):
             s3_bucket_name = s3_bucket_name.replace(str(number), "")
+        s3_bucket_name = s3_bucket_name.replace('xn--.', '').replace('-s3alias', '')
         if action == "deploy":
-            s3_bucket_name = self.deployment.s3.create_bucket(self.project_name)
+            s3_bucket_name = self.deployment.s3.create_bucket(s3_bucket_name)
             print("Created S3 bucket...")
 
         self.config.add_resource(
@@ -578,6 +588,16 @@ class Propheto:
         if action == "deploy":
             _model_filename = model_filepath.parts[-1]
             print(_model_filename)
+            # Upload a datetime stamped version of the model as well
+            _timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+            _split_model_filename, _split_model_filetype = _model_filename.split('.')
+            _timestamped_model_filename = f"models/{_split_model_filename}_{_timestamp}.{_split_model_filetype}"
+            _ = self.deployment.s3.upload_file(
+                project_name=self.project_name.replace(" ", ""),
+                source_path=model_filepath.as_posix(),
+                filename=_timestamped_model_filename,
+            )
+            # Upload main model
             s3_model_path = self.deployment.s3.upload_file(
                 project_name=self.project_name.replace(" ", ""),
                 source_path=model_filepath.as_posix(),
@@ -588,12 +608,13 @@ class Propheto:
         # UPLOAD LOGS
         if action == "deploy":
             log_path = Path(self.working_directory, "propheto-package", "logs")
-            s3_response = self.deployment.s3.upload_folder(
-                project_name=self.project_name.replace(" ", ""),
-                local_folder_path=log_path,
-                output_folder_path="logs",
-            )
-            print("Uploaded local logs")
+            if log_path.exists():
+                s3_response = self.deployment.s3.upload_folder(
+                    project_name=self.project_name.replace(" ", ""),
+                    local_folder_path=log_path,
+                    output_folder_path="logs",
+                )
+                print("Uploaded local logs")
 
         # GENERATE API CODE
         api_deployment_stage = 'dev'
@@ -760,6 +781,17 @@ class Propheto:
             # UPLOAD MODEL PACKAGE
             _model_filename = model_filepath.parts[-1]
             print(_model_filename)
+            # 
+            # Upload a datetime stamped version of the model as well
+            _timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+            _split_model_filename, _split_model_filetype = _model_filename.split('.')
+            _timestamped_model_filename = f"models/{_split_model_filename}_{_timestamp}.{_split_model_filetype}"
+            _ = self.deployment.s3.upload_file(
+                project_name=self.project_name.replace(" ", ""),
+                source_path=model_filepath.as_posix(),
+                filename=_timestamped_model_filename,
+            )
+            # Save the model
             s3_model_path = self.deployment.s3.upload_file(
                 project_name=project_name,
                 source_path=model_filepath.as_posix(),
